==> xcsoar-6.8.4.tar.xz: xcsoar-6.8.4/src/NMEA/ClimbInfo.hpp <==
/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2015 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

#ifndef XCSOAR_CLIMB_INFO_HPP
#define XCSOAR_CLIMB_INFO_HPP

#include "Math/fixed.hpp"

#include <type_traits>

/** Information about one thermal that was or is being climbed. */
struct OneClimbInfo
{
  /** Time when circling started. */
  fixed start_time;

  /**
   * Time when circling ended
   * (or current time stamp if circling has not ended yet).
   */
  fixed end_time;

  /** Time spent in this thermal [s]. */
  fixed duration;

  /** Altitude gained while in the thermal [m]. May be negative. */
  fixed gain;

  /** Average vertical speed in the thermal [m/s]. May be negative. */
  fixed lift_rate;

  void Clear();

  bool IsDefined() const {
    return positive(duration);
  }

  void CalculateDuration() {
    duration = end_time - start_time;
  }

  void CalculateLiftRate() {
    lift_rate = positive(duration)
      ? gain / duration
      : fixed(0);
  }

  void CalculateAll() {
    CalculateDuration();
    CalculateLiftRate();
  }
};

/** Derived climb data */
struct ClimbInfo
{
  OneClimbInfo current_thermal;

  OneClimbInfo last_thermal;

  /** Average vertical speed in the last thermals smoothed by low-pass-filter */
  fixed last_thermal_average_smooth;

  void Clear();
};

static_assert(std::is_trivial<ClimbInfo>::value, "type is not trivial");

#endif

==> xcsoar-7.11.tar.xz: xcsoar-7.11/src/NMEA/ClimbInfo.hpp <==
/*
Copyright_License {

  XCSoar Glide Computer - http://www.xcsoar.org/
  Copyright (C) 2000-2021 The XCSoar Project
  A detailed list of copyright holders can be found in the file "AUTHORS".

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}
*/

#ifndef XCSOAR_CLIMB_INFO_HPP
#define XCSOAR_CLIMB_INFO_HPP

#include <type_traits>

/** Information about one thermal that was or is being climbed. */
struct OneClimbInfo
{
  /** Time when circling started. */
  double start_time;

  /**
   * Time when circling ended
   * (or current time stamp if circling has not ended yet).
   */
  double end_time;

  /** Time spent in this thermal [s]. */
  double duration;

  /** Altitude gained while in the thermal [m]. May be negative. */
  double gain;

  /** Start altitude [m] */
  double start_altitude;

  /** Average vertical speed in the thermal [m/s]. May be negative. */
  double lift_rate;

  void Clear();

  bool IsDefined() const {
    return duration > 0;
  }

  void CalculateDuration() {
    duration = end_time - start_time;
  }

  void CalculateLiftRate() {
    lift_rate = IsDefined()
      ? gain / duration
      : 0.;
  }

  void CalculateAll() {
    CalculateDuration();
    CalculateLiftRate();
  }
};

/** Derived climb data */
struct ClimbInfo
{
  OneClimbInfo current_thermal;

  OneClimbInfo last_thermal;

  /** Average vertical speed in the last thermals smoothed by low-pass-filter */
  double last_thermal_average_smooth;

  void Clear();
};

static_assert(std::is_trivial<ClimbInfo>::value, "type is not trivial");

#endif
